@@ -2,6 +2,7 @@
  *
  * Copyright (C) 2008 William Jon McCann
  * Copyright (C) 2014 Michal Ratajsky <michal.ratajsky@gmail.com>
+ * Copyright (C) 2014-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,6 +43,7 @@ struct _GvcStreamStatusIconPrivate
         gchar           *display_name;
         MateMixerStreamControl *control;
         XAppStatusIcon  *status_icon;
+        GtkWidget       *status_menu_item;
 };
 
 enum
@@ -180,11 +182,7 @@ on_status_icon_button_release (XAppStatusIcon *icon,
 static void
 on_menu_mute_toggled (GtkMenuItem *item, GvcStreamStatusIcon *icon)
 {
-        gboolean is_muted;
-
-        is_muted = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (item));
-
-        mate_mixer_stream_control_set_mute (icon->priv->control, is_muted);
+        mate_mixer_stream_control_set_mute (icon->priv->control, !mate_mixer_stream_control_get_mute (icon->priv->control));
 }
 
 static void
@@ -221,10 +219,11 @@ create_popup_menu (GvcStreamStatusIcon *icon)
         GtkWidget *menu;
         GtkWidget *item;
         GtkWidget *image;
+        g_autofree char *label = NULL;
 
         menu = gtk_menu_new ();
 
-        /*Set up theme and transparency support*/
+        /* Set up theme and transparency support */
         GtkWidget *toplevel = gtk_widget_get_toplevel (menu);
         /* Fix any failures of compiz/other wm's to communicate with gtk for transparency */
         GdkScreen *screen = gtk_widget_get_screen(GTK_WIDGET(toplevel));
@@ -236,15 +235,26 @@ create_popup_menu (GvcStreamStatusIcon *icon)
         gtk_style_context_add_class(context,"gnome-panel-menu-bar");
         gtk_style_context_add_class(context,"mate-panel-menu-bar");
 
-        item = gtk_check_menu_item_new_with_mnemonic (_("_Mute"));
-        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
-                                        mate_mixer_stream_control_get_mute (icon->priv->control));
-        g_signal_connect (G_OBJECT (item),
-                          "toggled",
+        if (mate_mixer_stream_control_get_mute (icon->priv->control))
+        {
+                label = g_strdup_printf ("%s %s", _("Unmute"), icon->priv->display_name);
+                /* Set icon to medium*/
+                image = gtk_image_new_from_icon_name(icon->priv->icon_names[2], GTK_ICON_SIZE_MENU);
+        }
+        else
+        {
+                label = g_strdup_printf ("%s %s", _("Mute"), icon->priv->display_name);
+                /* Set icon to muted*/
+                image = gtk_image_new_from_icon_name(icon->priv->icon_names[0], GTK_ICON_SIZE_MENU);
+        }
+        icon->priv->status_menu_item = gtk_image_menu_item_new_with_mnemonic(label);
+        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (icon->priv->status_menu_item), image);
+        g_signal_connect (G_OBJECT (icon->priv->status_menu_item),
+                          "activate",
                           G_CALLBACK (on_menu_mute_toggled),
                           icon);
 
-        gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu), icon->priv->status_menu_item);
 
         item = gtk_image_menu_item_new_with_mnemonic (_("_Sound Preferences"));
         image = gtk_image_new_from_icon_name ("multimedia-volume-control",
@@ -362,12 +372,14 @@ on_dock_key_release (GtkWidget           *widget,
 static void
 update_icon (GvcStreamStatusIcon *icon)
 {
+        GtkWidget *menu_icon;
         guint                volume = 0;
         gdouble              decibel = 0;
         guint                normal = 0;
         gboolean             muted = FALSE;
         guint                n = 0;
         gchar               *markup;
+        gchar               *menu_label;
         const gchar         *description;
         MateMixerStreamControlFlags flags;
 
@@ -441,6 +453,22 @@ update_icon (GvcStreamStatusIcon *icon)
                                           description);
         }
 
+        if (muted)
+        {
+                menu_label = g_strdup_printf ("%s %s", _("Unmute"), icon->priv->display_name);
+                /* Set icon to medium*/
+                menu_icon = gtk_image_new_from_icon_name(icon->priv->icon_names[2], GTK_ICON_SIZE_MENU);
+        }
+        else
+        {
+                menu_label = g_strdup_printf ("%s %s", _("Mute"), icon->priv->display_name);
+                /* Set icon to muted*/
+                menu_icon = gtk_image_new_from_icon_name(icon->priv->icon_names[0], GTK_ICON_SIZE_MENU);
+        }
+
+        gtk_menu_item_set_label (GTK_MENU_ITEM (icon->priv->status_menu_item), menu_label);
+        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (icon->priv->status_menu_item), menu_icon);
+
         xapp_status_icon_set_tooltip_text (XAPP_STATUS_ICON (icon->priv->status_icon), markup);
 
         g_free (markup);
@@ -535,6 +563,10 @@ gvc_stream_status_icon_set_control (GvcStreamStatusIcon    *icon,
                                   G_CALLBACK (on_stream_control_mute_notify),
                                   icon);
 
+                if (xapp_status_icon_get_secondary_menu (icon->priv->status_icon) == NULL) {
+                        xapp_status_icon_set_secondary_menu (icon->priv->status_icon, GTK_MENU (create_popup_menu (icon)));
+                }
+
                 // XXX when no stream set some default icon and "unset" dock
                 update_icon (icon);
         }
@@ -702,7 +734,6 @@ gvc_stream_status_icon_init (GvcStreamStatusIcon *icon)
                                          GTK_ORIENTATION_VERTICAL);
 
         icon->priv->status_icon = xapp_status_icon_new ();
-        xapp_status_icon_set_secondary_menu (icon->priv->status_icon, GTK_MENU (create_popup_menu (icon)));
 
         g_signal_connect (icon->priv->status_icon,
                           "activate",
